@@ -29,10 +29,10 @@ typedef struct _file_head {
 typedef struct _file {
     file_head_t		file_head;
     slot_t		slots[1];
-} file_t;
+} speedy_file_t;
 
 #define FILE_ALLOC_CHUNK	512
-#define FILE_REV		'1'
+#define FILE_REV		2
 #define FILE_HEAD		(speedy_file_maddr->file_head)
 #define FILE_SLOTS		(speedy_file_maddr->slots)
 #define FILE_SLOT(member, n)	(FILE_SLOTS[SLOT_CHECK(n)-1].member)
@@ -50,11 +50,7 @@ typedef struct _file {
 #   define PREF_FD_FILE -1
 #endif
 
-#ifndef MAP_FAILED
-#   define MAP_FAILED (-1)
-#endif
-
-extern file_t *speedy_file_maddr;
+extern speedy_file_t *speedy_file_maddr;
 void speedy_file_fd_is_suspect();
 int speedy_file_size();
 void speedy_file_set_state(int new_state);
